@@ -23,6 +23,10 @@
 #ifndef SRC_DH_GEX_H_
 #define SRC_DH_GEX_H_
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 int ssh_client_dhgex_init(ssh_session session);
 void ssh_client_dhgex_remove_callbacks(ssh_session session);
 
@@ -30,4 +34,8 @@ void ssh_client_dhgex_remove_callbacks(ssh_session session);
 void ssh_server_dhgex_init(ssh_session session);
 #endif /* WITH_SERVER */
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif /* SRC_DH_GEX_H_ */
