@@ -103,6 +103,10 @@ int ssh_gcry_rand_range(bignum rnd, bignum max);
     } while(0)
 /* Helper functions for data conversions.  */
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /* Extract an MPI from the given s-expression SEXP named NAME which is
    encoded using INFORMAT and store it in a newly allocated ssh_string
    encoded using OUTFORMAT.  */
@@ -113,6 +117,10 @@ ssh_string ssh_sexp_extract_mpi(const gcry_sexp_t sexp,
 
 #define ssh_fips_mode() false
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif /* HAVE_LIBGCRYPT */
 
 #endif /* LIBGCRYPT_H_ */
