@@ -195,15 +195,24 @@ public:
     return ret;
   }
 
-  /** @brief Authenticate through the "keyboard-interactive" method.
-   * @param[in] The username to authenticate. You can specify NULL if ssh_option_set_username() has been used. You cannot try two different logins in a row.
-   * @param[in] Undocumented. Set it to NULL.
+  /**
+   * @brief Authenticate through the "keyboard-interactive" method.
+   *
+   * @param[in] username The username to authenticate. You can specify NULL if
+   *                     ssh_option_set_username() has been used. You cannot
+   *                     try two different logins in a row.
+   *
+   * @param[in] submethods Undocumented. Set it to NULL.
+   *
    * @throws SshException on error
-   * @returns SSH_AUTH_SUCCESS, SSH_AUTH_PARTIAL, SSH_AUTH_DENIED, SSH_AUTH_ERROR, SSH_AUTH_INFO, SSH_AUTH_AGAIN
+   *
+   * @returns SSH_AUTH_SUCCESS, SSH_AUTH_PARTIAL, SSH_AUTH_DENIED,
+   *          SSH_AUTH_ERROR, SSH_AUTH_INFO, SSH_AUTH_AGAIN
+   *
    * @see ssh_userauth_kbdint
    */
   int userauthKbdint(const char* username, const char* submethods){
-    int ret=ssh_userauth_kbdint(c_session,NULL,NULL);
+    int ret = ssh_userauth_kbdint(c_session, username, submethods);
     ssh_throw(ret);
     return ret;
   }
@@ -216,15 +225,25 @@ public:
     return ssh_userauth_kbdint_getnprompts(c_session);
   }
 
-  /** @brief Set the answer for a question from a message block..
-   * @param[in] index The number of the ith prompt.
-   * @param[in] The answer to give to the server. The answer MUST be encoded UTF-8. It is up to the server how to interpret the value and validate it. However, if you read the answer in some other encoding, you MUST convert it to UTF-8.
+  /**
+   * @brief Set the answer for a question from a message block.
+   *
+   * @param[in] index The index number of the prompt.
+   * @param[in] answer The answer to give to the server. The answer MUST be
+   *                   encoded UTF-8. It is up to the server how to interpret
+   *                   the value and validate it. However, if you read the
+   *                   answer in some other encoding, you MUST convert it to
+   *                   UTF-8.
+   *
    * @throws SshException on error
+   *
    * @returns 0 on success, < 0 on error
+   *
    * @see ssh_userauth_kbdint_setanswer
    */
-  int userauthKbdintSetAnswer(unsigned int i, const char* answer){
-    int ret=ssh_userauth_kbdint_setanswer(c_session, i, answer);
+  int userauthKbdintSetAnswer(unsigned int index, const char *answer)
+  {
+    int ret = ssh_userauth_kbdint_setanswer(c_session, index, answer);
     ssh_throw(ret);
     return ret;
   }
@@ -316,11 +335,10 @@ public:
    * @see ssh_get_issue_banner
    */
   std::string getIssueBanner(){
-    char *banner=ssh_get_issue_banner(c_session);
-    std::string ret;
-    if (banner)
-    {
-      ret= std::string(banner);
+    char *banner = ssh_get_issue_banner(c_session);
+    std::string ret = "";
+    if (banner != NULL) {
+      ret = std::string(banner);
       ::free(banner);
     }
     return ret;
@@ -351,13 +369,11 @@ public:
     return state;
   }
   void log(int priority, const char *format, ...){
-    char buffer[1024];
     va_list va;
 
     va_start(va, format);
-    vsnprintf(buffer, sizeof(buffer), format, va);
+    ssh_vlog(priority, "libsshpp", format, &va);
     va_end(va);
-    _ssh_log(priority, "libsshpp", "%s", buffer);
   }
 
   /** @brief copies options from a session to another
@@ -389,7 +405,7 @@ public:
    * @see ssh_write_knownhost
    */
   int writeKnownhost(){
-    int ret = ssh_write_knownhost(c_session);
+    int ret = ssh_session_update_known_hosts(c_session);
     ssh_throw(ret);
     return ret;
   }
@@ -438,9 +454,9 @@ private:
 class Channel {
   friend class Session;
 public:
-  Channel(Session &session){
-    channel=ssh_channel_new(session.getCSession());
-    this->session=&session;
+  Channel(Session &ssh_session){
+    channel = ssh_channel_new(ssh_session.getCSession());
+    this->session = &ssh_session;
   }
   ~Channel(){
     ssh_channel_free(channel);
@@ -507,7 +523,7 @@ public:
     return ssh_channel_is_open(channel) != 0;
   }
   int openForward(const char *remotehost, int remoteport,
-      const char *sourcehost=NULL, int localport=0){
+      const char *sourcehost, int localport=0){
     int err=ssh_channel_open_forward(channel,remotehost,remoteport,
         sourcehost, localport);
     ssh_throw(err);
@@ -614,8 +630,8 @@ public:
    * @param is_stderr write should be done on the stderr channel (server only)
    * @returns number of bytes written
    * @throws SshException in case of error
-   * @see channel_write
-   * @see channel_write_stderr
+   * @see ssh_channel_write
+   * @see ssh_channel_write_stderr
    */
   int write(const void *data, size_t len, bool is_stderr=false){
     int ret;
@@ -641,9 +657,9 @@ protected:
   ssh_channel channel;
 
 private:
-  Channel (Session &session, ssh_channel c_channel){
+  Channel (Session &ssh_session, ssh_channel c_channel){
     this->channel=c_channel;
-    this->session=&session;
+    this->session = &ssh_session;
   }
   /* No copy and no = operator */
   Channel(const Channel &);
@@ -653,7 +669,7 @@ private:
 
 inline Channel *Session::acceptForward(int timeout_ms){
     ssh_channel forward =
-        ssh_channel_accept_forward(c_session, timeout_ms, NULL);
+        ssh_channel_open_forward_port(c_session, timeout_ms, NULL, NULL, NULL);
     ssh_throw_null(c_session,forward);
     Channel *newchan = new Channel(*this,forward);
     return newchan;
