@@ -112,7 +112,7 @@ int ssh_packet_decrypt(ssh_session session,
         ssh_set_error(session,
                       SSH_FATAL,
                       "Cryptographic functions must be used on multiple of "
-                      "blocksize (received %" PRIdS ")",
+                      "blocksize (received %zu)",
                       encrypted_size);
         return SSH_ERROR;
     }
@@ -130,14 +130,15 @@ int ssh_packet_decrypt(ssh_session session,
     return 0;
 }
 
-unsigned char *ssh_packet_encrypt(ssh_session session, void *data, uint32_t len)
+unsigned char *ssh_packet_encrypt(ssh_session session, void *data, size_t len)
 {
   struct ssh_crypto_struct *crypto = NULL;
   struct ssh_cipher_struct *cipher = NULL;
   HMACCTX ctx = NULL;
   char *out = NULL;
-  int etm_packet_offset = 0;
-  unsigned int finallen, blocksize;
+  int etm_packet_offset = 0, rc;
+  unsigned int blocksize;
+  size_t finallen = DIGEST_MAX_LEN;
   uint32_t seq, lenfield_blocksize;
   enum ssh_hmac_e type;
   bool etm;
@@ -161,7 +162,7 @@ unsigned char *ssh_packet_encrypt(ssh_session session, void *data, uint32_t len)
 
   if ((len - lenfield_blocksize - etm_packet_offset) % blocksize != 0) {
       ssh_set_error(session, SSH_FATAL, "Cryptographic functions must be set"
-                    " on at least one blocksize (received %d)", len);
+                    " on at least one blocksize (received %zu)", len);
       return NULL;
   }
   out = calloc(1, len);
@@ -177,34 +178,62 @@ unsigned char *ssh_packet_encrypt(ssh_session session, void *data, uint32_t len)
             crypto->hmacbuf, session->send_seq);
       memcpy(data, out, len);
   } else {
-      ctx = hmac_init(crypto->encryptMAC, hmac_digest_len(type), type);
-      if (ctx == NULL) {
-        SAFE_FREE(out);
-        return NULL;
-      }
+      if (type != SSH_HMAC_NONE) {
+          ctx = hmac_init(crypto->encryptMAC, hmac_digest_len(type), type);
+          if (ctx == NULL) {
+              SAFE_FREE(out);
+              return NULL;
+          }
 
-      if (!etm) {
-          hmac_update(ctx, (unsigned char *)&seq, sizeof(uint32_t));
-          hmac_update(ctx, data, len);
-          hmac_final(ctx, crypto->hmacbuf, &finallen);
+          if (!etm) {
+              rc = hmac_update(ctx, (unsigned char *)&seq, sizeof(uint32_t));
+              if (rc != 1) {
+                  SAFE_FREE(out);
+                  return NULL;
+              }
+              rc = hmac_update(ctx, data, len);
+              if (rc != 1) {
+                  SAFE_FREE(out);
+                  return NULL;
+              }
+              rc = hmac_final(ctx, crypto->hmacbuf, &finallen);
+              if (rc != 1) {
+                  SAFE_FREE(out);
+                  return NULL;
+              }
+          }
       }
 
       cipher->encrypt(cipher, (uint8_t*)data + etm_packet_offset, out, len - etm_packet_offset);
       memcpy((uint8_t*)data + etm_packet_offset, out, len - etm_packet_offset);
 
-      if (etm) {
-          PUSH_BE_U32(data, 0, len - etm_packet_offset);
-          hmac_update(ctx, (unsigned char *)&seq, sizeof(uint32_t));
-          hmac_update(ctx, data, len);
-          hmac_final(ctx, crypto->hmacbuf, &finallen);
-      }
+      if (type != SSH_HMAC_NONE) {
+          if (etm) {
+              PUSH_BE_U32(data, 0, len - etm_packet_offset);
+              rc = hmac_update(ctx, (unsigned char *)&seq, sizeof(uint32_t));
+              if (rc != 1) {
+                  SAFE_FREE(out);
+                  return NULL;
+              }
+              rc = hmac_update(ctx, data, len);
+              if (rc != 1) {
+                  SAFE_FREE(out);
+                  return NULL;
+              }
+              rc = hmac_final(ctx, crypto->hmacbuf, &finallen);
+              if (rc != 1) {
+                  SAFE_FREE(out);
+                  return NULL;
+              }
+          }
 #ifdef DEBUG_CRYPTO
-      ssh_log_hexdump("mac: ", data, len);
-      if (finallen != hmac_digest_len(type)) {
-          printf("Final len is %d\n", finallen);
-      }
-      ssh_log_hexdump("Packet hmac", crypto->hmacbuf, hmac_digest_len(type));
+          ssh_log_hexdump("mac: ", data, len);
+          if (finallen != hmac_digest_len(type)) {
+              printf("Final len is %zu\n", finallen);
+          }
+          ssh_log_hexdump("Packet hmac", crypto->hmacbuf, hmac_digest_len(type));
 #endif
+      }
   }
   explicit_bzero(out, len);
   SAFE_FREE(out);
@@ -212,17 +241,6 @@ unsigned char *ssh_packet_encrypt(ssh_session session, void *data, uint32_t len)
   return crypto->hmacbuf;
 }
 
-static int secure_memcmp(const void *s1, const void *s2, size_t n)
-{
-    int rc = 0;
-    const unsigned char *p1 = s1;
-    const unsigned char *p2 = s2;
-    for (; n > 0; --n) {
-        rc |= *p1++ ^ *p2++;
-    }
-    return (rc != 0);
-}
-
 /**
  * @internal
  *
@@ -242,42 +260,71 @@ int ssh_packet_hmac_verify(ssh_session session,
                            uint8_t *mac,
                            enum ssh_hmac_e type)
 {
-  struct ssh_crypto_struct *crypto = NULL;
-  unsigned char hmacbuf[DIGEST_MAX_LEN] = {0};
-  HMACCTX ctx;
-  unsigned int hmaclen;
-  uint32_t seq;
+    struct ssh_crypto_struct *crypto = NULL;
+    unsigned char hmacbuf[DIGEST_MAX_LEN] = {0};
+    HMACCTX ctx;
+    size_t hmaclen = DIGEST_MAX_LEN;
+    uint32_t seq;
+    int cmp;
+    int rc;
 
-  /* AEAD types have no mac checking */
-  if (type == SSH_HMAC_AEAD_POLY1305 ||
-      type == SSH_HMAC_AEAD_GCM) {
-      return SSH_OK;
-  }
+    /* AEAD types have no mac checking */
+    if (type == SSH_HMAC_AEAD_POLY1305 ||
+        type == SSH_HMAC_AEAD_GCM) {
+        return SSH_OK;
+    }
 
-  crypto = ssh_packet_get_current_crypto(session, SSH_DIRECTION_IN);
-  if (crypto == NULL) {
-      return SSH_ERROR;
-  }
+    crypto = ssh_packet_get_current_crypto(session,
+                                           SSH_DIRECTION_IN);
+    if (crypto == NULL) {
+        return SSH_ERROR;
+    }
 
-  ctx = hmac_init(crypto->decryptMAC, hmac_digest_len(type), type);
-  if (ctx == NULL) {
-    return -1;
-  }
+    ctx = hmac_init(crypto->decryptMAC,
+                    hmac_digest_len(type),
+                    type);
+    if (ctx == NULL) {
+        return SSH_ERROR;
+    }
 
-  seq = htonl(session->recv_seq);
+    seq = htonl(session->recv_seq);
 
-  hmac_update(ctx, (unsigned char *) &seq, sizeof(uint32_t));
-  hmac_update(ctx, data, len);
-  hmac_final(ctx, hmacbuf, &hmaclen);
+    rc = hmac_update(ctx,
+                     (unsigned char *) &seq,
+                     sizeof(uint32_t));
+    if (rc != 1) {
+        return SSH_ERROR;
+    }
+    rc = hmac_update(ctx,
+                     data,
+                     len);
+    if (rc != 1) {
+        return SSH_ERROR;
+    }
+    rc = hmac_final(ctx,
+                    hmacbuf,
+                    &hmaclen);
+    if (rc != 1) {
+        return SSH_ERROR;
+    }
 
 #ifdef DEBUG_CRYPTO
-  ssh_log_hexdump("received mac",mac,hmaclen);
-  ssh_log_hexdump("Computed mac",hmacbuf,hmaclen);
-  ssh_log_hexdump("seq",(unsigned char *)&seq,sizeof(uint32_t));
+    ssh_log_hexdump("received mac",
+                    mac,
+                    hmaclen);
+    ssh_log_hexdump("Computed mac",
+                    hmacbuf,
+                    hmaclen);
+    ssh_log_hexdump("seq",
+                    (unsigned char *)&seq,
+                    sizeof(uint32_t));
 #endif
-  if (secure_memcmp(mac, hmacbuf, hmaclen) == 0) {
-    return 0;
-  }
+    cmp = secure_memcmp(mac,
+                        hmacbuf,
+                        hmaclen);
+    if (cmp == 0) {
+        return SSH_OK;
+    }
 
-  return -1;
+    return SSH_ERROR;
 }
