@@ -58,7 +58,6 @@
 
 /* Buffer size maximum is 256M */
 #define SFTP_PACKET_SIZE_MAX 0x10000000
-#define SFTP_BUFFER_SIZE_MAX 16384
 
 struct sftp_ext_struct {
   uint32_t count;
@@ -229,40 +228,42 @@ error:
 }
 
 #ifdef WITH_SERVER
-sftp_session sftp_server_new(ssh_session session, ssh_channel chan){
-  sftp_session sftp = NULL;
+sftp_session
+sftp_server_new(ssh_session session, ssh_channel chan)
+{
+    sftp_session sftp = NULL;
 
-  sftp = calloc(1, sizeof(struct sftp_session_struct));
-  if (sftp == NULL) {
-    ssh_set_error_oom(session);
-    return NULL;
-  }
+    sftp = calloc(1, sizeof(struct sftp_session_struct));
+    if (sftp == NULL) {
+        ssh_set_error_oom(session);
+        return NULL;
+    }
 
-  sftp->read_packet = calloc(1, sizeof(struct sftp_packet_struct));
-  if (sftp->read_packet == NULL) {
-    goto error;
-  }
+    sftp->read_packet = calloc(1, sizeof(struct sftp_packet_struct));
+    if (sftp->read_packet == NULL) {
+        goto error;
+    }
 
-  sftp->read_packet->payload = ssh_buffer_new();
-  if (sftp->read_packet->payload == NULL) {
-    goto error;
-  }
+    sftp->read_packet->payload = ssh_buffer_new();
+    if (sftp->read_packet->payload == NULL) {
+        goto error;
+    }
 
-  sftp->session = session;
-  sftp->channel = chan;
+    sftp->session = session;
+    sftp->channel = chan;
 
-  return sftp;
+    return sftp;
 
 error:
-  ssh_set_error_oom(session);
-  if (sftp->read_packet != NULL) {
-    if (sftp->read_packet->payload != NULL) {
-      SSH_BUFFER_FREE(sftp->read_packet->payload);
+    ssh_set_error_oom(session);
+    if (sftp->read_packet != NULL) {
+        if (sftp->read_packet->payload != NULL) {
+            SSH_BUFFER_FREE(sftp->read_packet->payload);
+        }
+        SAFE_FREE(sftp->read_packet);
     }
-    SAFE_FREE(sftp->read_packet);
-  }
-  SAFE_FREE(sftp);
-  return NULL;
+    SAFE_FREE(sftp);
+    return NULL;
 }
 
 int sftp_server_init(sftp_session sftp){
@@ -386,11 +387,11 @@ void sftp_free(sftp_session sftp)
     SAFE_FREE(sftp);
 }
 
-ssize_t sftp_packet_write(sftp_session sftp, uint8_t type, ssh_buffer payload)
+int sftp_packet_write(sftp_session sftp, uint8_t type, ssh_buffer payload)
 {
     uint8_t header[5] = {0};
     uint32_t payload_size;
-    ssize_t size;
+    int size;
     int rc;
 
     /* Add size of type */
@@ -415,7 +416,7 @@ ssize_t sftp_packet_write(sftp_session sftp, uint8_t type, ssh_buffer payload)
 
     if ((uint32_t)size != ssh_buffer_get_len(payload)) {
         SSH_LOG(SSH_LOG_PACKET,
-                "Had to write %d bytes, wrote only %zd",
+                "Had to write %d bytes, wrote only %d",
                 ssh_buffer_get_len(payload),
                 size);
     }
@@ -425,7 +426,8 @@ ssize_t sftp_packet_write(sftp_session sftp, uint8_t type, ssh_buffer payload)
 
 sftp_packet sftp_packet_read(sftp_session sftp)
 {
-    uint8_t buffer[SFTP_BUFFER_SIZE_MAX];
+    uint8_t tmpbuf[4];
+    uint8_t *buffer = NULL;
     sftp_packet packet = sftp->read_packet;
     uint32_t size;
     int nread;
@@ -459,7 +461,7 @@ sftp_packet sftp_packet_read(sftp_session sftp)
         int s;
 
         // read from channel until 4 bytes have been read or an error occurs
-        s = ssh_channel_read(sftp->channel, buffer + nread, 4 - nread, 0);
+        s = ssh_channel_read(sftp->channel, tmpbuf + nread, 4 - nread, 0);
         if (s < 0) {
             goto error;
         } else if (s == 0) {
@@ -476,7 +478,7 @@ sftp_packet sftp_packet_read(sftp_session sftp)
         }
     } while (nread < 4);
 
-    size = PULL_BE_U32(buffer, 0);
+    size = PULL_BE_U32(tmpbuf, 0);
     if (size == 0 || size > SFTP_PACKET_SIZE_MAX) {
         ssh_set_error(sftp->session, SSH_FATAL, "Invalid sftp packet size!");
         sftp_set_error(sftp, SSH_FX_FAILURE);
@@ -484,7 +486,7 @@ sftp_packet sftp_packet_read(sftp_session sftp)
     }
 
     do {
-        nread = ssh_channel_read(sftp->channel, buffer, 1, 0);
+        nread = ssh_channel_read(sftp->channel, tmpbuf, 1, 0);
         if (nread < 0) {
             goto error;
         } else if (nread == 0) {
@@ -499,34 +501,28 @@ sftp_packet sftp_packet_read(sftp_session sftp)
         }
     } while (nread < 1);
 
-    packet->type = buffer[0];
+    packet->type = tmpbuf[0];
 
     /* Remove the packet type size */
     size -= sizeof(uint8_t);
 
-    nread = ssh_buffer_allocate_size(packet->payload, size);
-    if (nread < 0) {
+    /* Allocate the receive buffer from payload */
+    buffer = ssh_buffer_allocate(packet->payload, size);
+    if (buffer == NULL) {
         ssh_set_error_oom(sftp->session);
         sftp_set_error(sftp, SSH_FX_FAILURE);
         goto error;
     }
     while (size > 0 && size < SFTP_PACKET_SIZE_MAX) {
-        nread = ssh_channel_read(sftp->channel,
-                             buffer,
-                             sizeof(buffer) > size ? size : sizeof(buffer),
-                             0);
+        nread = ssh_channel_read(sftp->channel, buffer, size, 0);
         if (nread < 0) {
             /* TODO: check if there are cases where an error needs to be set here */
             goto error;
         }
 
         if (nread > 0) {
-            rc = ssh_buffer_add_data(packet->payload, buffer, nread);
-            if (rc != 0) {
-                ssh_set_error_oom(sftp->session);
-                sftp_set_error(sftp, SSH_FX_FAILURE);
-                goto error;
-            }
+            buffer += nread;
+            size -= nread;
         } else { /* nread == 0 */
             /* Retry the reading unless the remote was closed */
             is_eof = ssh_channel_is_eof(sftp->channel);
@@ -538,8 +534,6 @@ sftp_packet sftp_packet_read(sftp_session sftp)
                 goto error;
             }
         }
-
-        size -= nread;
     }
 
     return packet;
@@ -872,7 +866,7 @@ static int sftp_enqueue(sftp_session sftp, sftp_message msg) {
 }
 
 /*
- * Pulls of a message from the queue based on the ID.
+ * Pulls a message from the queue based on the ID.
  * Returns NULL if no message has been found.
  */
 static sftp_message sftp_dequeue(sftp_session sftp, uint32_t id){
@@ -1108,7 +1102,7 @@ sftp_dir sftp_opendir(sftp_session sftp, const char *path)
 /*
  * Parse the attributes from a payload from some messages. It is coded on
  * baselines from the protocol version 4.
- * This code is more or less dead but maybe we need it in future.
+ * This code is more or less dead but maybe we will need it in the future.
  */
 static sftp_attributes sftp_parse_attr_4(sftp_session sftp, ssh_buffer buf,
     int expectnames) {
@@ -1296,7 +1290,7 @@ static char *sftp_parse_longname(const char *longname,
     size_t len, field = 0;
 
     p = longname;
-    /* Find the beginning of the field which is specified by sftp_longanme_field_e. */
+    /* Find the beginning of the field which is specified by sftp_longname_field_e. */
     while(field != longname_field) {
         if(isspace(*p)) {
             field++;
@@ -2009,7 +2003,7 @@ ssize_t sftp_read(sftp_file handle, void *buf, size_t count) {
       if (datalen > count) {
         ssh_set_error(sftp->session, SSH_FATAL,
             "Received a too big DATA packet from sftp server: "
-            "%" PRIdS " and asked for %" PRIdS,
+            "%zu and asked for %zu",
             datalen, count);
         SSH_STRING_FREE(datastring);
         return -1;
@@ -2131,7 +2125,7 @@ int sftp_async_read(sftp_file file, void *data, uint32_t size, uint32_t id){
       if (ssh_string_len(datastring) > size) {
         ssh_set_error(sftp->session, SSH_FATAL,
             "Received a too big DATA packet from sftp server: "
-            "%" PRIdS " and asked for %u",
+            "%zu and asked for %u",
             ssh_string_len(datastring), size);
         SSH_STRING_FREE(datastring);
         return SSH_ERROR;
@@ -2184,8 +2178,8 @@ ssize_t sftp_write(sftp_file file, const void *buf, size_t count) {
     sftp_set_error(sftp, SSH_FX_FAILURE);
     return -1;
   }
-  packetlen=ssh_buffer_get_len(buffer);
   len = sftp_packet_write(file->sftp, SSH_FXP_WRITE, buffer);
+  packetlen=ssh_buffer_get_len(buffer);
   SSH_BUFFER_FREE(buffer);
   if (len < 0) {
     return -1;
