@@ -9,42 +9,20 @@
 #define __PKD_CLIENT_H__
 
 #include "config.h"
+#include "tests_config.h"
 
 /* OpenSSH */
 
-#define OPENSSH_BINARY "ssh"
+#define OPENSSH_BINARY SSH_EXECUTABLE
 #define OPENSSH_KEYGEN "ssh-keygen"
 
-#define OPENSSH_HOSTKEY_ALGOS_DEFAULT "ssh-ed25519,rsa-sha2-512,rsa-sha2-256,ssh-rsa"
-#define OPENSSH_PKACCEPTED_DEFAULT    "ssh-ed25519,rsa-sha2-512,rsa-sha2-256,ssh-rsa,ssh-rsa-cert-v01@openssh.com,ssh-ed25519-cert-v01@openssh.com,rsa-sha2-256-cert-v01@openssh.com,rsa-sha2-512-cert-v01@openssh.com"
-
-#if       HAVE_ECC
-#define OPENSSH_HOSTKEY_ALGOS_ECDSA   ",ecdsa-sha2-nistp256,ecdsa-sha2-nistp384,ecdsa-sha2-nistp521"
-#define OPENSSH_PKACCEPTED_ECDSA      ",ecdsa-sha2-nistp256,ecdsa-sha2-nistp384,ecdsa-sha2-nistp521,ecdsa-sha2-nistp256-cert-v01@openssh.com,ecdsa-sha2-nistp384-cert-v01@openssh.com,ecdsa-sha2-nistp521-cert-v01@openssh.com"
-#else  /* HAVE_ECC */
-#define OPENSSH_HOSTKEY_ALGOS_ECDSA   ""
-#define OPENSSH_PKACCEPTED_ECDSA      ""
-#endif /* HAVE_ECC */
-
-#if       HAVE_DSA
-#define OPENSSH_HOSTKEY_ALGOS_DSA     ",ssh-dss"
-#define OPENSSH_PKACCEPTED_DSA        ",ssh-dss,ssh-dss-cert-v01@openssh.com"
-#else  /* HAVE_DSA */
-#define OPENSSH_HOSTKEY_ALGOS_DSA     ""
-#define OPENSSH_PKACCEPTED_DSA        ""
-#endif /* HAVE_DSA */
-
 #define OPENSSH_HOSTKEY_ALGOS \
   "-o HostKeyAlgorithms="        \
-  OPENSSH_HOSTKEY_ALGOS_DEFAULT  \
-  OPENSSH_HOSTKEY_ALGOS_ECDSA    \
-  OPENSSH_HOSTKEY_ALGOS_DSA
+  OPENSSH_KEYS
 
 #define OPENSSH_PKACCEPTED_TYPES \
   "-o PubkeyAcceptedKeyTypes="  \
-  OPENSSH_PKACCEPTED_DEFAULT    \
-  OPENSSH_PKACCEPTED_ECDSA      \
-  OPENSSH_PKACCEPTED_DSA
+  OPENSSH_KEYS
 
 #define OPENSSH_CMD_START(hostkey_algos) \
     OPENSSH_BINARY " "                  \
