@@ -1231,6 +1231,7 @@ void torture_teardown_sshd_server(void **state)
 }
 #endif /* SSHD_EXECUTABLE */
 
+#ifdef WITH_PKCS11_URI
 void torture_setup_tokens(const char *temp_dir,
                           const char *filename,
                           const char object_name[],
@@ -1240,15 +1241,18 @@ void torture_setup_tokens(const char *temp_dir,
     int rc;
 
     snprintf(token_setup_start_cmd, sizeof(token_setup_start_cmd),
-             "%s/tests/pkcs11/setup-softhsm-tokens.sh %s %s %s %s",
+             "%s/tests/pkcs11/setup-softhsm-tokens.sh %s %s %s %s %s",
              BINARYDIR,
              temp_dir,
              filename,
-             object_name, load_public);
+             object_name,
+             load_public,
+             SOFTHSM2_LIBRARY);
 
     rc = system(token_setup_start_cmd);
     assert_return_code(rc, errno);
 }
+#endif /* WITH_PKCS11_URI */
 
 char *torture_make_temp_dir(const char *template)
 {
@@ -1629,12 +1633,14 @@ void torture_reset_config(ssh_session session)
     memset(session->opts.options_seen, 0, sizeof(session->opts.options_seen));
 }
 
-#if ((defined _WIN32) || (defined _WIN64)) && (defined USE_ATTRIBUTE_WEAK)
+#if defined(HAVE_WEAK_ATTRIBUTE) && defined(TORTURE_SHARED)
 __attribute__((weak)) int torture_run_tests(void)
 {
-    fail();
+    fail_msg("torture_run_tests from shared library called");
+
+    return -1;
 }
-#endif
+#endif /* defined(HAVE_WEAK_ATTRIBUTE) && defined(TORTURE_SHARED) */
 
 int main(int argc, char **argv) {
     struct argument_s arguments;
